(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     61236,       1395]
NotebookOptionsPosition[     56917,       1320]
NotebookOutlinePosition[     57251,       1335]
CellTagsIndexPosition[     57208,       1332]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["StateMint Example", "Title",
 CellChangeTimes->{{3.749517748387988*^9, 3.7495177578584023`*^9}, {
  3.752074679813195*^9, 3.7520746861317587`*^9}, {3.7531991258476343`*^9, 
  3.753199126839426*^9}},ExpressionUUID->"d3849ac4-c012-46c7-b86b-\
5e1ba9703926"],

Cell["\<\
This notebook will focus on how to interact with StateMint in a Mathematica \
notebook.\
\>", "Text",
 CellChangeTimes->{{3.749517768393689*^9, 3.74951781569664*^9}, {
  3.752074695638905*^9, 3.752074729204295*^9}, {3.753199100036727*^9, 
  3.753199101053008*^9}},ExpressionUUID->"dee37b68-b003-47bb-83aa-\
34f50c8d94d0"],

Cell["Before we begin we need to import the StateMint package.", "Text",
 CellChangeTimes->{{3.749517831842804*^9, 3.74951784732657*^9}, {
  3.752074736641693*^9, 3.752074738766007*^9}, {3.753199097196782*^9, 
  3.753199098364512*^9}},ExpressionUUID->"736874a7-447e-420a-8fe9-\
247fc95dd459"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"NotebookDirectory", "[", "]"}], "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"<<", "StateMint`"}]}], "Input",
 CellChangeTimes->{{3.749517850744534*^9, 3.749517904428472*^9}, {
   3.752074859105585*^9, 3.752074860517879*^9}, 3.752076465465398*^9, {
   3.753199093907756*^9, 
   3.753199094921145*^9}},ExpressionUUID->"4ae703ef-2121-4e4a-857f-\
262964453034"],

Cell[TextData[{
 "Now that StateMint has been imported we can start working on a problem. In \
this case we will be working on the problem set forth in the ",
 ButtonBox["tutorial",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["https://github.com/CameronDevine/StateMint/blob/master/tutorial.md"],
     None},
  ButtonNote->
   "https://github.com/CameronDevine/StateMint/blob/master/tutorial.md"],
 ". This problem consists of a motor powering a pump through a flexible \
shaft. This pump pushes water through an elbow with a known resistance and \
out into the atmosphere. A diagram of the physical system can be seen below."
}], "Text",
 CellChangeTimes->{{3.7495179168731947`*^9, 3.7495179367746696`*^9}, {
  3.749517973149335*^9, 3.749518010190508*^9}, {3.752074918533415*^9, 
  3.7520749200987577`*^9}, {3.752074980552095*^9, 3.7520749808721447`*^9}, {
  3.7531991046505327`*^9, 
  3.753199106022131*^9}},ExpressionUUID->"9a3a49fd-e894-426a-9ec8-\
6ca841f06f7b"],

Cell[BoxData[
 GraphicsBox[
  TagBox[RasterBox[CompressedData["
1:eJztnQd4FEUbx/936RUCofcivfcOAtIEETAKIlKEAEpRQEBEqUqQIkVEVNCg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    "], {{0, 222}, {861, 0}}, {0, 255},
    ColorFunction->GrayLevel],
   BoxForm`ImageTag["Byte", ColorSpace -> "Grayscale", Interleaving -> None],
   Selectable->False],
  DefaultBaseStyle->"ImageGraphics",
  ImageSizeRaw->{861, 222},
  PlotRange->{{0, 861}, {0, 222}}]], "Input",ExpressionUUID->"049bf644-cb53-\
4e85-b08b-365e0b6afffe"],

Cell["\<\
The following equations for this system were found in the tutorial .\
\>", "Text",
 CellChangeTimes->{{3.7495182918191347`*^9, 3.749518296746046*^9}, {
  3.752075750946992*^9, 
  3.7520757511794786`*^9}},ExpressionUUID->"558df971-96b2-4ea3-88fa-\
d91934866a49"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"inVars", "=", 
   RowBox[{"{", 
    RowBox[{"Vs", "[", "t", "]"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"primaryVars", " ", "=", " ", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"tk", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"vR", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"vL", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"i1", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"w2", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"w3", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"P4", "[", "t", "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"QR", "[", "t", "]"}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"elementalEqs", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"tk", "'"}], "[", "t", "]"}], "\[Equal]", 
      RowBox[{"kt", " ", 
       RowBox[{"wk", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vR", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"R", " ", 
       RowBox[{"iR", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"vL", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"L", " ", 
       RowBox[{
        RowBox[{"iL", "'"}], "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     
     RowBox[{
      RowBox[{"i1", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"-", "Kv"}], " ", 
       RowBox[{"t2", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"w2", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"Kv", " ", 
       RowBox[{"v1", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"w3", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"Q4", "[", "t", "]"}], "/", 
       RowBox[{"-", "DD"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"P4", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"t3", "[", "t", "]"}], "/", "DD"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"QR", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"PR", "[", "t", "]"}], "/", "Rf"}]}]}], "\[IndentingNewLine]", 
    "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"constraintEqs", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"iL", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"i1", "[", "t", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"iR", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"i1", "[", "t", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"t2", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"-", 
       RowBox[{"tk", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"t3", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"tk", "[", "t", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Q4", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"QR", "[", "t", "]"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"v1", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"Vs", "[", "t", "]"}], "-", 
       RowBox[{"vR", "[", "t", "]"}], "-", 
       RowBox[{"vL", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"wk", "[", "t", "]"}], "\[Equal]", 
      RowBox[{
       RowBox[{"w2", "[", "t", "]"}], "-", 
       RowBox[{"w3", "[", "t", "]"}]}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PR", "[", "t", "]"}], "\[Equal]", 
      RowBox[{"P4", "[", "t", "]"}]}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"outExps", "=", 
   RowBox[{"{", 
    RowBox[{"QR", "[", "t", "]"}], "}"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.7495183069606*^9, 3.749518538126053*^9}, {
  3.7495186172483473`*^9, 3.749518640291297*^9}, {3.752075776698476*^9, 
  3.752075926490069*^9}, {3.752076585839171*^9, 3.752076586211344*^9}},
 CellLabel->"In[16]:=",ExpressionUUID->"0c27e491-35b1-4cda-acb1-b9a5c08763cd"],

Cell["\<\
Using the following command, the state equations can be found.\
\>", "Text",
 CellChangeTimes->{{3.749518799424985*^9, 3.749518810605731*^9}, {
  3.7520759769970713`*^9, 3.752075987481391*^9}, {3.752076366531571*^9, 
  3.752076369261776*^9}},ExpressionUUID->"243d6832-afeb-4a11-b0b2-\
10d3ab515d3d"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"stateSolution", "=", 
   RowBox[{
    RowBox[{"stateEquations", "[", 
     RowBox[{
     "inVars", ",", "primaryVars", ",", "elementalEqs", ",", 
      "constraintEqs"}], "]"}], "//", "Flatten"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"stateVars", " ", "=", " ", 
   RowBox[{"\"\<state variables\>\"", "/.", "stateSolution"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"stateEqList", " ", "=", " ", 
   RowBox[{"\"\<RHS\>\"", "/.", "stateSolution"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"\"\<state equations\>\"", "/.", "stateSolution"}], "//", 
  "TableForm"}]}], "Input",
 CellChangeTimes->{{3.7018797195836763`*^9, 3.7018797204767036`*^9}, {
   3.701879810803969*^9, 3.701879844544981*^9}, {3.7018942050184517`*^9, 
   3.701894566874161*^9}, 3.701898020551093*^9, {3.701903282327601*^9, 
   3.701903283500523*^9}, {3.701911200698804*^9, 3.701911202601316*^9}, {
   3.701915682054678*^9, 3.701915720675168*^9}, {3.701915759772274*^9, 
   3.7019157927706213`*^9}, {3.701915923602686*^9, 3.701915923900078*^9}, 
   3.7019209193048553`*^9, 3.701920950145496*^9, {3.701921306764739*^9, 
   3.701921308673313*^9}, {3.701921408184596*^9, 3.7019214156809187`*^9}, 
   3.7019220747800493`*^9, {3.701922189786292*^9, 3.701922218088579*^9}, {
   3.701922248592429*^9, 3.701922251876824*^9}, 3.701922322852202*^9, {
   3.701922383801173*^9, 3.7019224315654707`*^9}, {3.701922463501308*^9, 
   3.7019224895050097`*^9}, 3.7019225472984467`*^9, {3.701922594846767*^9, 
   3.701922598092758*^9}, {3.701922662563005*^9, 3.701922735877761*^9}, {
   3.7019231046383343`*^9, 3.701923105684675*^9}, {3.7019639715923243`*^9, 
   3.7019639718841667`*^9}, {3.7019642654040737`*^9, 3.701964352167139*^9}, {
   3.701964387110903*^9, 3.7019644147887087`*^9}, {3.702757419429731*^9, 
   3.7027575039883213`*^9}, {3.7511302149210863`*^9, 3.751130222225973*^9}, 
   3.751131245052647*^9, {3.751758020590618*^9, 3.7517580314940643`*^9}, {
   3.751758096034569*^9, 3.751758150178234*^9}, {3.751858834585458*^9, 
   3.751858867892303*^9}, {3.7518589221835537`*^9, 3.7518590012649117`*^9}, {
   3.751859049763453*^9, 3.75185909971*^9}, {3.7518592622618513`*^9, 
   3.75185926422972*^9}, {3.751859306929682*^9, 3.751859367871738*^9}, {
   3.7518594346930857`*^9, 3.751859464138123*^9}, {3.751859792301899*^9, 
   3.7518598121041403`*^9}, {3.7518598489196587`*^9, 3.751859923477129*^9}, 
   3.75186099185213*^9, 3.751937475686306*^9},
 CellLabel->"In[8]:=",
 CellID->1449591737,ExpressionUUID->"68f14f0a-68cc-47c8-b1bd-da3eb46ddfb3"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{
        SuperscriptBox["tk", "\[Prime]",
         MultilineFunction->None], "[", "t", "]"}], "\[Equal]", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{"kt", "-", 
            RowBox[{
             SuperscriptBox["DD", "2"], " ", "kt", " ", 
             SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}], ")"}], " ", 
          RowBox[{"tk", "[", "t", "]"}]}], 
         RowBox[{
          SuperscriptBox["DD", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            RowBox[{"kt", " ", 
             SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]], 
        "+", 
        FractionBox[
         RowBox[{"kt", " ", "Kv", " ", 
          RowBox[{"Vs", "[", "t", "]"}]}], 
         RowBox[{"1", "+", 
          RowBox[{"kt", " ", 
           SuperscriptBox["Kv", "2"], " ", "L"}]}]]}]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{{3.7518593365167*^9, 3.751859368149544*^9}, 
   3.751859469567102*^9, 3.751859641381487*^9, 3.751859929862179*^9, 
   3.751859981942204*^9, {3.751861010775159*^9, 3.75186103657938*^9}, 
   3.752076373756929*^9, 3.752076484471745*^9},
 CellLabel->
  "Out[11]//TableForm=",ExpressionUUID->"9255e3e8-baa2-48e7-93e4-\
d02a205005c6"]
}, Open  ]],

Cell["In a similar way, the output equations can be found.", "Text",
 CellChangeTimes->{{3.7520766271595907`*^9, 
  3.7520766467332363`*^9}},ExpressionUUID->"fee207ac-a92c-40f9-8955-\
3a0ef126ebcd"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"outEqList", "=", 
   RowBox[{"outEquations", "[", 
    RowBox[{"inVars", ",", "stateVars", ",", "outExps", ",", 
     RowBox[{"elementalEqs", "~", "Join", "~", "constraintEqs"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"outEqList", "//", 
   RowBox[{
    RowBox[{"Thread", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"y", "[", "i", "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "1", ",", 
          RowBox[{"Length", "[", "outExps", "]"}]}], "}"}]}], "]"}], 
      "\[Equal]", "#"}], "]"}], "&"}]}], "//", "TableForm"}]}], "Input",
 CellChangeTimes->{{3.75115407229806*^9, 3.7511540907068577`*^9}, {
   3.7511541283727303`*^9, 3.751154145140655*^9}, {3.751154185947949*^9, 
   3.751154186454541*^9}, {3.75115440553708*^9, 3.751154408241644*^9}, {
   3.751157061478198*^9, 3.75115706228808*^9}, {3.751157258647379*^9, 
   3.751157263470278*^9}, {3.751157318329653*^9, 3.751157382599204*^9}, 
   3.751243546881419*^9, 3.7512446518094807`*^9, {3.751758233817813*^9, 
   3.75175824703872*^9}},
 CellLabel->"In[21]:=",
 CellID->168493274,ExpressionUUID->"1e3b22c3-818c-4665-ba95-13871edd5826"],

Cell[BoxData[
 TagBox[
  TagBox[GridBox[{
     {
      RowBox[{
       RowBox[{"y", "[", "1", "]"}], "\[Equal]", 
       FractionBox[
        RowBox[{"tk", "[", "t", "]"}], 
        RowBox[{"DD", " ", "Rf"}]]}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.5599999999999999]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}],
   Column],
  Function[BoxForm`e$, 
   TableForm[BoxForm`e$]]]], "Output",
 CellChangeTimes->{
  3.751154146341402*^9, {3.7511541874520807`*^9, 3.75115421035389*^9}, 
   3.751154409289606*^9, 3.751155063159431*^9, 3.7511551578703537`*^9, {
   3.751155631171876*^9, 3.7511556669733973`*^9}, 3.751156831153225*^9, {
   3.7511570444961233`*^9, 3.751157063011318*^9}, 3.7511573845757217`*^9, 
   3.7511576929574337`*^9, 3.751237248545403*^9, 3.75123731846103*^9, 
   3.751237558009218*^9, 3.75124126512782*^9, 3.751241341691312*^9, 
   3.7512414602930183`*^9, {3.7512415890389347`*^9, 3.751241614550478*^9}, 
   3.751243391757332*^9, {3.751243525864511*^9, 3.7512435473077707`*^9}, 
   3.7512436090180597`*^9, 3.751243754645124*^9, 3.751243860169832*^9, 
   3.751244182055914*^9, 3.751244399162704*^9, 3.751244486514542*^9, 
   3.751244523619996*^9, 3.7512445593242817`*^9, {3.751244640162719*^9, 
   3.7512446526642714`*^9}, 3.7512450693463917`*^9, 3.751245129916429*^9, 
   3.751245199944006*^9, 3.7517581589332533`*^9, 3.751758247484844*^9, 
   3.7518117541350803`*^9, 3.7518120056910477`*^9, {3.7518120796408854`*^9, 
   3.751812087500362*^9}, {3.751812187588044*^9, 3.751812209829607*^9}, 
   3.751824464419462*^9, 3.751827255574244*^9, 3.751859641437621*^9, 
   3.7518599517115088`*^9, 3.751859982075001*^9, {3.751861010824903*^9, 
   3.75186103667798*^9}, 3.752076506380468*^9, 3.7520765899367*^9},
 CellLabel->
  "Out[22]//TableForm=",ExpressionUUID->"6a1f7b24-08fa-4272-bc18-\
ecbf92b7613a"]
}, Open  ]],

Cell["\<\
Linearize the state and output equations about the origin. Note that they \
equations are already linear, but this step is useful for finding the linear \
operators.\
\>", "Text",
 CellChangeTimes->{{3.7027577023090067`*^9, 3.7027577064193897`*^9}, {
  3.702757795179463*^9, 3.702757797848589*^9}, {3.702758517676444*^9, 
  3.702758517969734*^9}, {3.751157404122614*^9, 3.751157407793623*^9}, {
  3.752076666333744*^9, 3.7520767184530783`*^9}},
 CellID->1243308122,ExpressionUUID->"58bca072-25c9-412d-92e7-5062006193ff"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"abe", "=", 
   RowBox[{"linearizeState", "[", 
    RowBox[{"inVars", ",", "stateVars", ",", "stateEqList"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"cdf", "=", 
   RowBox[{"linearizeOutput", "[", 
    RowBox[{"inVars", ",", "stateVars", ",", "outEqList"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "e"}], "}"}], " ", "=", " ", "abe"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"{", 
    RowBox[{"c", ",", "d", ",", "f"}], "}"}], " ", "=", " ", "cdf"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<A\>\"", ",", "\"\<B\>\"", ",", "\"\<E\>\""}], "}"}], 
        "[", 
        RowBox[{"[", "i", "]"}], "]"}], "<>", "\"\< = \>\""}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"abe", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "//", "MatrixForm"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "\[IndentingNewLine]", "]"}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Table", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Print", "[", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<C\>\"", ",", "\"\<D\>\"", ",", "\"\<F\>\""}], "}"}], 
        "[", 
        RowBox[{"[", "i", "]"}], "]"}], "<>", "\"\< = \>\""}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"cdf", "[", 
        RowBox[{"[", "i", "]"}], "]"}], "//", "MatrixForm"}]}], "]"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"{", 
     RowBox[{"i", ",", "1", ",", "3"}], "}"}]}], "\[IndentingNewLine]", "]"}],
   ";"}]}], "Input",
 CellChangeTimes->{{3.7027439116175957`*^9, 3.702744033267743*^9}, {
   3.702744268313121*^9, 3.702744305447536*^9}, {3.702756777004896*^9, 
   3.70275678344866*^9}, {3.702756831990849*^9, 3.702756860110464*^9}, {
   3.702757530029274*^9, 3.702757693348208*^9}, {3.70275831097122*^9, 
   3.702758346655436*^9}, {3.702758404323063*^9, 3.7027584657658863`*^9}, {
   3.702759733356806*^9, 3.702760026609954*^9}, {3.70276007934348*^9, 
   3.702760085749899*^9}, {3.702760162502097*^9, 3.7027602713332367`*^9}, {
   3.7031743380777597`*^9, 3.703174365043808*^9}, {3.7032126025366993`*^9, 
   3.703212608677622*^9}, {3.7275822132423773`*^9, 3.727582355849462*^9}, 
   3.751154035567876*^9, {3.751157216290072*^9, 3.751157229742868*^9}, {
   3.751157416529111*^9, 3.751157421062249*^9}, {3.7511576066364393`*^9, 
   3.751157656881137*^9}, {3.7511577043529177`*^9, 3.7511577145152082`*^9}, {
   3.7512372971265383`*^9, 3.7512373336944847`*^9}, {3.751237381927462*^9, 
   3.751237410777441*^9}, {3.75207677021564*^9, 3.75207683862414*^9}},
 CellLabel->"In[24]:=",
 CellID->1263721144,ExpressionUUID->"776bde41-1ddb-4711-9f56-dcb55f78bb22"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"A = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {
        FractionBox[
         RowBox[{"kt", "-", 
          RowBox[{
           SuperscriptBox["DD", "2"], " ", "kt", " ", 
           SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}], 
         RowBox[{
          SuperscriptBox["DD", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            RowBox[{"kt", " ", 
             SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["A = ", 
   MatrixForm[{{$CellContext`DD^(-2) (
        1 + $CellContext`kt $CellContext`Kv^2 $CellContext`L)^(-1) \
$CellContext`Rf^(-1) ($CellContext`kt - $CellContext`DD^2 $CellContext`kt \
$CellContext`Kv^2 $CellContext`R $CellContext`Rf)}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.752076842423059*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"5a584e1e-1820-4ebb-9b5b-e42761396b5c"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"B = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {
        FractionBox[
         RowBox[{"kt", " ", "Kv"}], 
         RowBox[{"1", "+", 
          RowBox[{"kt", " ", 
           SuperscriptBox["Kv", "2"], " ", "L"}]}]]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["B = ", 
   MatrixForm[{{$CellContext`kt $CellContext`Kv/(
      1 + $CellContext`kt $CellContext`Kv^2 $CellContext`L)}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.752076842435828*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"cf9bae7a-6ac1-4757-a1f1-03f12fc238a0"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"E = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["E = ", 
   MatrixForm[{{0}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.752076842445363*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"21af23fc-80c2-4ab0-862c-09a11983b55d"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"C = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {
        FractionBox["1", 
         RowBox[{"DD", " ", "Rf"}]]}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["C = ", 
   MatrixForm[{{$CellContext`DD^(-1)/$CellContext`Rf}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.752076842450294*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"8cbf0e3d-ede6-49e2-af98-02eac40d7f4a"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"D = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["D = ", 
   MatrixForm[{{0}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.7520768424550543`*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"fe6560ac-d336-46b8-9697-b73087b050c9"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"F = \"\>", "\[InvisibleSpace]", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[BoxForm`e$]]]}],
  SequenceForm["F = ", 
   MatrixForm[{{0}}]],
  Editable->False]], "Print",
 CellChangeTimes->{
  3.75124360906213*^9, 3.7512437547528667`*^9, 3.751243860273984*^9, 
   3.751244182134783*^9, 3.751244399232733*^9, 3.751244486581511*^9, 
   3.75124455936265*^9, 3.7512446402133417`*^9, 3.751244670741263*^9, 
   3.751245069381319*^9, 3.751245129946912*^9, 3.7512451999957867`*^9, 
   3.751758253230949*^9, 3.751811754203699*^9, 3.751812005763603*^9, {
   3.751812079703279*^9, 3.751812087609988*^9}, {3.751812187660625*^9, 
   3.751812209901985*^9}, 3.751824464483783*^9, 3.751827255619883*^9, 
   3.7518596414861507`*^9, 3.751859982209362*^9, {3.751861010877256*^9, 
   3.7518610367788982`*^9}, 3.752076842459983*^9},
 CellLabel->
  "During evaluation of \
In[24]:=",ExpressionUUID->"495b5093-ca5c-49d9-96fc-a915ecf2516d"]
}, Open  ]]
}, Open  ]],

Cell["\<\
The results can also be viewed in other languages such as LaTeX,\
\>", "Text",
 CellChangeTimes->{{3.749519004775931*^9, 
  3.7495190192240267`*^9}},ExpressionUUID->"a6fbf9b4-2792-4e36-ba8a-\
ccb4c87d8b63"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a", "//", "TeXForm"}]], "Input",
 CellChangeTimes->{{3.749519022571344*^9, 3.749519061905691*^9}},
 CellLabel->"In[30]:=",ExpressionUUID->"17be4c9f-eade-4683-8f67-61704fc7aa5a"],

Cell["\<\
\\left(
\\begin{array}{c}
 \\frac{\\text{kt}-\\text{DD}^2 \\text{kt} \\text{Kv}^2 R
   \\text{Rf}}{\\text{DD}^2 \\left(\\text{kt} L \\text{Kv}^2+1\\right)
   \\text{Rf}} \\\\
\\end{array}
\\right)\
\>", "Output",
 CellChangeTimes->{3.7495190296879463`*^9, 3.749519062396727*^9, 
  3.749568910749227*^9, 3.749569418643159*^9, 3.749569500845234*^9, 
  3.749582797308194*^9, 3.74958311222946*^9, 3.7520768651458063`*^9},
 CellLabel->
  "Out[30]//TeXForm=",ExpressionUUID->"cb47caff-2dec-4068-acc0-2c02afc2be95"]
}, Open  ]],

Cell["\<\
Other Mathematica functions can be used to manipulate the equations. At this \
point let\[CloseCurlyQuote]s solve the differential equation,\
\>", "Text",
 CellChangeTimes->{{3.749519149185121*^9, 3.7495191771852207`*^9}, {
  3.749519858089747*^9, 
  3.7495198881295958`*^9}},ExpressionUUID->"7c2257b8-5345-427b-8414-\
02f236e5c37e"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"input", "=", 
   RowBox[{
    RowBox[{"Vs", "[", "t", "]"}], "\[Rule]", "12"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ic", "=", 
   RowBox[{
    RowBox[{"tk", "[", "0", "]"}], "\[Equal]", "0"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"x", "=", 
  RowBox[{
   RowBox[{"tk", "[", "t", "]"}], "/.", 
   RowBox[{"DSolve", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"\"\<state equations\>\"", "/.", "stateSolution"}], "/.", 
        "input"}], ",", " ", "ic"}], "}"}], ",", " ", 
     RowBox[{"tk", "[", "t", "]"}], ",", "t"}], "]"}]}]}]}], "Input",
 CellChangeTimes->{{3.749519529890267*^9, 3.749519567628292*^9}, {
   3.749519643758359*^9, 3.7495196439553413`*^9}, {3.7495196843080177`*^9, 
   3.74951968913516*^9}, {3.749519734532897*^9, 3.7495197482954817`*^9}, {
   3.749519783984531*^9, 3.749519807261735*^9}, {3.7495199427543917`*^9, 
   3.749519957745386*^9}, {3.7495200237438507`*^9, 3.749520034386744*^9}, {
   3.749568934362719*^9, 3.749568934856111*^9}, {3.749569010211591*^9, 
   3.749569012214755*^9}, {3.749569206821939*^9, 3.749569237961897*^9}, {
   3.749569269713147*^9, 3.749569304121559*^9}, {3.7495693851038933`*^9, 
   3.749569397931593*^9}, {3.7495695391596518`*^9, 3.749569594638369*^9}, {
   3.749569632841178*^9, 3.7495696522401342`*^9}, 3.749583104603222*^9, 
   3.752076933535617*^9},
 CellLabel->"In[31]:=",ExpressionUUID->"b813a373-ab67-475d-9c08-1e779c4d0900"],

Cell[BoxData[
 RowBox[{"{", 
  FractionBox[
   RowBox[{"12", " ", 
    SuperscriptBox["DD", "2"], " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"kt", " ", 
         SuperscriptBox["Kv", "2"], " ", "R", " ", "t"}], 
        RowBox[{"1", "+", 
         RowBox[{"kt", " ", 
          SuperscriptBox["Kv", "2"], " ", "L"}]}]]}], "+", 
      FractionBox[
       RowBox[{"kt", " ", "t"}], 
       RowBox[{
        SuperscriptBox["DD", "2"], " ", 
        RowBox[{"(", 
         RowBox[{"1", "+", 
          RowBox[{"kt", " ", 
           SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]]}]], 
    " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       FractionBox[
        RowBox[{"kt", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "1"}], "+", 
           RowBox[{
            SuperscriptBox["DD", "2"], " ", 
            SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}], ")"}], " ", 
         "t"}], 
        RowBox[{
         SuperscriptBox["DD", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"kt", " ", 
            SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]]]}], 
     ")"}], " ", "Kv", " ", "Rf"}], 
   RowBox[{
    RowBox[{"-", "1"}], "+", 
    RowBox[{
     SuperscriptBox["DD", "2"], " ", 
     SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}]], "}"}]], "Output",
 CellChangeTimes->{{3.74956891083479*^9, 3.7495689365990133`*^9}, 
   3.7495690134771338`*^9, {3.749569213036499*^9, 3.74956923852315*^9}, {
   3.749569282893488*^9, 3.7495693047109623`*^9}, {3.749569392912613*^9, 
   3.749569419114038*^9}, 3.749569503312047*^9, {3.7495695448224287`*^9, 
   3.7495695950276814`*^9}, 3.7495696533957453`*^9, 3.7495828035001907`*^9, 
   3.7495831122641983`*^9, 3.7520769397088346`*^9},
 CellLabel->"Out[33]=",ExpressionUUID->"e62cb264-133c-4773-828b-8a1eb53701a5"]
}, Open  ]],

Cell["\<\
Using this solution of the state variable we can find the output of the \
system. \
\>", "Text",
 CellChangeTimes->{{3.749582965024211*^9, 
  3.7495829872337437`*^9}},ExpressionUUID->"67a44709-e6a4-4f51-8602-\
6ff9ff737d88"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"y", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"c", " ", "x"}], "+", 
    RowBox[{"d", " ", 
     RowBox[{"Vs", "[", "t", "]"}]}]}], "/.", "input"}]}]], "Input",
 CellChangeTimes->{{3.749582839412788*^9, 3.749582885064273*^9}, 
   3.749582950691036*^9, {3.7495831067165527`*^9, 3.749583108056682*^9}},
 CellLabel->"In[34]:=",ExpressionUUID->"cd4cc7c3-9449-491d-aa55-4fd158a24118"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   FractionBox[
    RowBox[{"12", " ", "DD", " ", 
     SuperscriptBox["\[ExponentialE]", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"kt", " ", 
          SuperscriptBox["Kv", "2"], " ", "R", " ", "t"}], 
         RowBox[{"1", "+", 
          RowBox[{"kt", " ", 
           SuperscriptBox["Kv", "2"], " ", "L"}]}]]}], "+", 
       FractionBox[
        RowBox[{"kt", " ", "t"}], 
        RowBox[{
         SuperscriptBox["DD", "2"], " ", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"kt", " ", 
            SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]]}]], 
     " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "1"}], "+", 
       SuperscriptBox["\[ExponentialE]", 
        FractionBox[
         RowBox[{"kt", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", "1"}], "+", 
            RowBox[{
             SuperscriptBox["DD", "2"], " ", 
             SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}], ")"}], " ", 
          "t"}], 
         RowBox[{
          SuperscriptBox["DD", "2"], " ", 
          RowBox[{"(", 
           RowBox[{"1", "+", 
            RowBox[{"kt", " ", 
             SuperscriptBox["Kv", "2"], " ", "L"}]}], ")"}], " ", "Rf"}]]]}], 
      ")"}], " ", "Kv"}], 
    RowBox[{
     RowBox[{"-", "1"}], "+", 
     RowBox[{
      SuperscriptBox["DD", "2"], " ", 
      SuperscriptBox["Kv", "2"], " ", "R", " ", "Rf"}]}]], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{{3.749582839780129*^9, 3.7495828453277903`*^9}, 
   3.749582887988096*^9, 3.749582951134042*^9, 3.749583112287949*^9, 
   3.7520769516953297`*^9},
 CellLabel->"Out[34]=",ExpressionUUID->"cfe8d805-f0a7-4dca-8212-51017bd03c54"]
}, Open  ]],

Cell["\<\
In order to plot the response let\[CloseCurlyQuote]s substitute in values for \
the variables.\
\>", "Text",
 CellChangeTimes->{{3.7495199706846523`*^9, 
  3.749519997969543*^9}},ExpressionUUID->"df11b5dc-1675-47f0-b118-\
946a9edb87e8"],

Cell[BoxData[
 RowBox[{
  RowBox[{"values", "=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"R", "\[Rule]", "5"}], ",", "\[IndentingNewLine]", 
     RowBox[{"L", "\[Rule]", "0.1"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Kv", "\[Rule]", "1000"}], ",", "\[IndentingNewLine]", 
     RowBox[{"kt", "\[Rule]", "10"}], ",", "\[IndentingNewLine]", 
     RowBox[{"DD", "\[Rule]", "0.0015"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Rf", "\[Rule]", 
      RowBox[{"4", " ", 
       SuperscriptBox["10", "6"]}]}]}], "\[IndentingNewLine]", "}"}]}], 
  ";"}]], "Input",
 CellChangeTimes->{{3.749519180606724*^9, 3.749519233042643*^9}, {
  3.749519346266242*^9, 3.749519353363784*^9}, {3.749519901930931*^9, 
  3.749519905134046*^9}, {3.749569600063818*^9, 3.749569600779901*^9}, {
  3.749583043523979*^9, 3.749583044436204*^9}},
 CellLabel->"In[35]:=",ExpressionUUID->"9d3da45f-86a7-433e-8b33-41228f0431c7"],

Cell["\<\
Given these values the solution of the differential equation is,\
\>", "Text",
 CellChangeTimes->{{3.749519413579241*^9, 3.7495194467639503`*^9}, {
  3.749520004518491*^9, 
  3.7495200165526447`*^9}},ExpressionUUID->"5e7a952d-5cce-40af-a864-\
ade91f770dee"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"y", "/.", "values"}]], "Input",
 CellChangeTimes->{{3.749520040929762*^9, 3.749520045003749*^9}, 
   3.749582995706922*^9},
 CellLabel->"In[36]:=",ExpressionUUID->"8c5995dc-2652-478e-99a2-63f315b47f36"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"4.0000000888888914`*^-7", " ", 
    SuperscriptBox["\[ExponentialE]", 
     RowBox[{
      RowBox[{"-", "49.99994888894`"}], " ", "t"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "1"}], "+", 
      SuperscriptBox["\[ExponentialE]", 
       RowBox[{"49.99994888894`", " ", "t"}]]}], ")"}]}], "}"}], 
  "}"}]], "Output",
 CellChangeTimes->{3.7495689109067287`*^9, 3.749569024939619*^9, 
  3.749569419210929*^9, 3.7495695147819843`*^9, 3.749569603974784*^9, 
  3.749569662194606*^9, 3.749582996161759*^9, 3.74958304631566*^9, 
  3.749583112324799*^9, 3.752076961084675*^9},
 CellLabel->"Out[36]=",ExpressionUUID->"47eca807-a078-44f1-b9af-008cb516da37"]
}, Open  ]],

Cell["Finally this output can be plotted to view the response.", "Text",
 CellChangeTimes->{{3.749520049304844*^9, 3.7495200727028103`*^9}, {
  3.749583168186613*^9, 
  3.749583168591362*^9}},ExpressionUUID->"0188c3f4-6353-459f-b1ef-\
21d4c0a55814"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Plot", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"y", "/.", "values"}], "/.", 
    RowBox[{"t", "\[Rule]", "tt"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"tt", ",", "0", ",", "0.15"}], "}"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.7495200737455673`*^9, 3.749520093401775*^9}, {
   3.749520137840425*^9, 3.7495201452997847`*^9}, {3.749569056711115*^9, 
   3.749569071425085*^9}, 3.74958308700711*^9},
 CellLabel->"In[37]:=",ExpressionUUID->"f78f2d21-516c-4dcc-9e4e-fe24821ae2b9"],

Cell[BoxData[
 GraphicsBox[{{{}, {}, 
    TagBox[
     {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], Opacity[
      1.], LineBox[CompressedData["
1:eJwBkQNu/CFib1JlAgAAADgAAAACAAAAbeIWEvxogj+pCyvdiWuDPtRltGrE
q4I/k8ydE2Cngz4tLlHkt6eIPypvrH6kR4g+9ioSf4Mljz/h2LFM/ZaMPkGf
ClI0mpI/DBh3EyoFkD5AQx71kuKVPxgc/1uFn5E+6EwjIZ4bmT8eVSJqN/eS
PvJhSRI2HZw/PPkJBIwKlD40kQEUul+fP86+/BTLCpU+7WVtbWU1oT/g5TWw
3deVPhS2UhVEs6I/pkZOU2SFlj5YE4HFmFGkP7T1B7IqJ5c+TfY/2DPUpT8r
Z1TJtqiXPl7mR/NEd6c/lSXvqCchmD5EichSrBKpP6QdN5MRhpg+27HZFFqS
qj+4ddOa7daYPo7nM999Mqw/7/f1azsimT7yoh4M6LatP0eWk/+CXpk+cmtS
Qchbrz85RQ/HfpaZPmNzf11/fLA/bGnrm1/FmT7l853LPT2xP9SpAfXr6pk+
9frgPTcOsj8VHt9v2g2aPt5EbOFT0bI/erBoA84pmj4x6DOnm5CzP8Cok3Nz
QZo+EhIgcR5gtD/R8dSrfFeaPsx+VGzEIbU/css8byFpmj4Ucq1rpfO1P9Zf
aAOHeZo+xr5CjbHBtj8bu0o3Q4eaPlFOIODggbc/A2r7QESSmj5qZCI3S1K4
P9hRrq+CnJo+W71sv9gUuT8L1VOYtaSaPrdv82mR07k/mSXn36Wrmj6hqJ4Y
haK6P2sN3Ecespo+YySS+Jtjuz9ZCM7TTLeaPrMmqtztNLw/pRHSbh68mj7c
awryYvi8P/2Db2T5v5o+bwqnKQO4vT9x5wslPMOaPpAvaGXeh74/4+QkvUXG
mj6Kl3HS3Em/P3Z9/T60yJo+CsPPIQsOwD96y+KH9sqaPgTnhGs9dcA/DL7+
1NrMmj5qLN5NgdXAPzF4Ycdezpo+F7VJsuI9wT8AvLbHx8+aPjBfWa9Vn8E/
gY7rq+jQmj7+NYc93v7BPyuz1hnd0Zo+E1DHTYRmwj9wn0bxwNKaPpWLq/Y7
x8I/YcCOYnfTmj4+1wpq+83CP0NzEyOD05o+6CJq3brUwj9+v8bEjtOaPjq6
KMQ54sI/TJP7rKXTmj7e6KWRN/3CP0xQQRnS05o+hzQFBfcDwz9L3+Pr3NOa
PjCAZHi2CsM/I9gkoufTmj6CFyNfNRjDP/P3qrr805o+1K7hRbQlwz85/R1l
EdSaPidGoCwzM8M/LkK8oyXUmj77F7oY
       "]]},
     Annotation[#, "Charting`Private`Tag$7881#1"]& ]}, {}, {}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, 1.446908145155786*^-7},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, 
     Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, 
     Charting`ScaledFrameTicks[{Identity, Identity}]}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImagePadding->All,
  Method->{
   "DefaultBoundaryStyle" -> Automatic, "DefaultMeshStyle" -> 
    AbsolutePointSize[6], "ScalingFunctions" -> None, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->{{0, 0.15}, {1.446908145155786*^-7, 3.997787734059209*^-7}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.7495689112251997`*^9, {3.74956905185963*^9, 3.749569081599256*^9}, 
   3.749569438215549*^9, 3.749569608692129*^9, 3.749569670461959*^9, 
   3.749583000749013*^9, 3.749583048241681*^9, {3.749583087304582*^9, 
   3.74958311237635*^9}, 3.752076963914777*^9},
 CellLabel->"Out[37]=",ExpressionUUID->"510b1ba5-d67a-4052-a80a-052d4a576990"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{681, 711},
WindowMargins->{{158, Automatic}, {1, Automatic}},
FrontEndVersion->"11.3 for Linux x86 (64-bit) (March 6, 2018)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 261, 4, 98, "Title",ExpressionUUID->"d3849ac4-c012-46c7-b86b-5e1ba9703926"],
Cell[844, 28, 331, 7, 58, "Text",ExpressionUUID->"dee37b68-b003-47bb-83aa-34f50c8d94d0"],
Cell[1178, 37, 292, 4, 35, "Text",ExpressionUUID->"736874a7-447e-420a-8fe9-247fc95dd459"],
Cell[1473, 43, 432, 10, 55, "Input",ExpressionUUID->"4ae703ef-2121-4e4a-857f-262964453034"],
Cell[1908, 55, 973, 19, 127, "Text",ExpressionUUID->"9a3a49fd-e894-426a-9ec8-6ca841f06f7b"],
Cell[2884, 76, 14314, 239, 144, "Input",ExpressionUUID->"049bf644-cb53-4e85-b08b-365e0b6afffe"],
Cell[17201, 317, 271, 6, 35, "Text",ExpressionUUID->"558df971-96b2-4ea3-88fa-d91934866a49"],
Cell[17475, 325, 4068, 103, 745, "Input",ExpressionUUID->"0c27e491-35b1-4cda-acb1-b9a5c08763cd"],
Cell[21546, 430, 309, 6, 35, "Text",ExpressionUUID->"243d6832-afeb-4a11-b0b2-10d3ab515d3d"],
Cell[CellGroupData[{
Cell[21880, 440, 2588, 46, 147, "Input",ExpressionUUID->"68f14f0a-68cc-47c8-b1bd-da3eb46ddfb3",
 CellID->1449591737],
Cell[24471, 488, 1794, 50, 75, "Output",ExpressionUUID->"9255e3e8-baa2-48e7-93e4-d02a205005c6"]
}, Open  ]],
Cell[26280, 541, 198, 3, 35, "Text",ExpressionUUID->"fee207ac-a92c-40f9-8955-3a0ef126ebcd"],
Cell[CellGroupData[{
Cell[26503, 548, 1202, 27, 101, "Input",ExpressionUUID->"1e3b22c3-818c-4665-ba95-13871edd5826",
 CellID->168493274],
Cell[27708, 577, 2127, 44, 60, "Output",ExpressionUUID->"6a1f7b24-08fa-4272-bc18-ecbf92b7613a"]
}, Open  ]],
Cell[29850, 624, 530, 9, 58, "Text",ExpressionUUID->"58bca072-25c9-412d-92e7-5062006193ff",
 CellID->1243308122],
Cell[CellGroupData[{
Cell[30405, 637, 3025, 74, 331, "Input",ExpressionUUID->"776bde41-1ddb-4711-9f56-dcb55f78bb22",
 CellID->1263721144],
Cell[CellGroupData[{
Cell[33455, 715, 2099, 48, 42, "Print",ExpressionUUID->"5a584e1e-1820-4ebb-9b5b-e42761396b5c"],
Cell[35557, 765, 1734, 40, 33, "Print",ExpressionUUID->"cf9bae7a-6ac1-4757-a1f1-03f12fc238a0"],
Cell[37294, 807, 1477, 34, 25, "Print",ExpressionUUID->"21af23fc-80c2-4ab0-862c-09a11983b55d"],
Cell[38774, 843, 1571, 36, 33, "Print",ExpressionUUID->"8cbf0e3d-ede6-49e2-af98-02eac40d7f4a"],
Cell[40348, 881, 1479, 34, 25, "Print",ExpressionUUID->"fe6560ac-d336-46b8-9697-b73087b050c9"],
Cell[41830, 917, 1477, 34, 25, "Print",ExpressionUUID->"495b5093-ca5c-49d9-96fc-a915ecf2516d"]
}, Open  ]]
}, Open  ]],
Cell[43334, 955, 216, 5, 35, "Text",ExpressionUUID->"a6fbf9b4-2792-4e36-ba8a-ccb4c87d8b63"],
Cell[CellGroupData[{
Cell[43575, 964, 201, 3, 31, "Input",ExpressionUUID->"17be4c9f-eade-4683-8f67-61704fc7aa5a"],
Cell[43779, 969, 518, 13, 164, "Output",ExpressionUUID->"cb47caff-2dec-4068-acc0-2c02afc2be95"]
}, Open  ]],
Cell[44312, 985, 343, 7, 58, "Text",ExpressionUUID->"7c2257b8-5345-427b-8414-02f236e5c37e"],
Cell[CellGroupData[{
Cell[44680, 996, 1489, 33, 124, "Input",ExpressionUUID->"b813a373-ab67-475d-9c08-1e779c4d0900"],
Cell[46172, 1031, 1990, 54, 105, "Output",ExpressionUUID->"e62cb264-133c-4773-828b-8a1eb53701a5"]
}, Open  ]],
Cell[48177, 1088, 234, 6, 35, "Text",ExpressionUUID->"67a44709-e6a4-4f51-8602-6ff9ff737d88"],
Cell[CellGroupData[{
Cell[48436, 1098, 402, 9, 31, "Input",ExpressionUUID->"cd4cc7c3-9449-491d-aa55-4fd158a24118"],
Cell[48841, 1109, 1772, 52, 105, "Output",ExpressionUUID->"cfe8d805-f0a7-4dca-8212-51017bd03c54"]
}, Open  ]],
Cell[50628, 1164, 246, 6, 35, "Text",ExpressionUUID->"df11b5dc-1675-47f0-b118-946a9edb87e8"],
Cell[50877, 1172, 927, 18, 209, "Input",ExpressionUUID->"9d3da45f-86a7-433e-8b33-41228f0431c7"],
Cell[51807, 1192, 267, 6, 35, "Text",ExpressionUUID->"5e7a952d-5cce-40af-a864-ade91f770dee"],
Cell[CellGroupData[{
Cell[52099, 1202, 226, 4, 31, "Input",ExpressionUUID->"8c5995dc-2652-478e-99a2-63f315b47f36"],
Cell[52328, 1208, 722, 17, 40, "Output",ExpressionUUID->"47eca807-a078-44f1-b9af-008cb516da37"]
}, Open  ]],
Cell[53065, 1228, 249, 4, 35, "Text",ExpressionUUID->"0188c3f4-6353-459f-b1ef-21d4c0a55814"],
Cell[CellGroupData[{
Cell[53339, 1236, 501, 11, 31, "Input",ExpressionUUID->"f78f2d21-516c-4dcc-9e4e-fe24821ae2b9"],
Cell[53843, 1249, 3046, 67, 220, "Output",ExpressionUUID->"510b1ba5-d67a-4052-a80a-052d4a576990"]
}, Open  ]]
}, Open  ]]
}
]
*)

